package moe.fuqiuluo.shamrock.ui.theme

import android.app.Activity
import android.os.Build
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.darkColorScheme
import androidx.compose.material3.dynamicDarkColorScheme
import androidx.compose.material3.dynamicLightColorScheme
import androidx.compose.material3.lightColorScheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.SideEffect
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.toArgb
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalView
import androidx.core.view.WindowCompat

lateinit var GlobalColor: ThemeColor

private val DarkColorScheme = darkColorScheme(
    primary = ThemeColor.ColorTabSelected,
    secondary = Color(0xFF000000),
    tertiary = Color(0xFF000000),
    surface = Color(0xFF000000),
)

private val LightColorScheme = lightColorScheme(
    primary = ThemeColor.ColorTabSelected,
    secondary = Color(0xFFCCC2DC),
    tertiary = Color(0xFFFFFBFE),
    surface = Color(0xFFFFFBFE),
)

@Composable
fun ShamrockTheme(
    darkTheme: Boolean = isSystemInDarkTheme(),
    // Dynamic color is available on Android 12+
    dynamicColor: Boolean = true,
    content: @Composable () -> Unit
) {
    val colorScheme = when {
        dynamicColor && Build.VERSION.SDK_INT >= Build.VERSION_CODES.S -> {
            val context = LocalContext.current
            if (darkTheme) {
                GlobalColor = DarkColor
                dynamicDarkColorScheme(context)
            } else {
                GlobalColor = LightColor
                dynamicLightColorScheme(context)
            }
        }

        darkTheme -> {
            GlobalColor = DarkColor
            DarkColorScheme
        }
        else -> {
            GlobalColor = LightColor
            LightColorScheme
        }
    }
    val view = LocalView.current
    if (!view.isInEditMode) {
        SideEffect {
            val window = (view.context as Activity).window
            window.statusBarColor = colorScheme.primary.toArgb()
            WindowCompat.getInsetsController(window, view).isAppearanceLightStatusBars = darkTheme
        }
    }

    MaterialTheme(
        colorScheme = colorScheme,
        typography = Typography,
        content = content
    )
}